@@ -1,5 +1,6 @@
 #!/usr/bin/env python
 
+# Version 0.0.3
 
 """
 
@@ -22,13 +23,40 @@ snarf /snarf/ vt.
 
 """
 
+
 # Python, wxPython stuff:
 from wxPython.wx import *
 from wxPython.lib.filebrowsebutton import FileBrowseButton
 from wxPython.html import *
-from xml.parsers import expat
+
+
 import os, os.path, string, shutil, time, sys, urllib, UserDict, regex, posix
 
+
+
+# expat is on python.org, but doesn't come with all installations, make a check and warning for it:
+# Not included on Mandrake 7.0
+
+py2 = sys.version[0] == '2'
+
+try:
+    if py2:
+        from xml.parsers import expat
+        parsermodule = expat
+    else:
+        from xml.parsers import pyexpat
+        parsermodule = pyexpat
+    haveXML = true
+except ImportError:
+    haveXML = false
+
+
+if not haveXML:
+    print "This demo requires the XML package.  \n" \
+          "See http://www.python.org/sigs/xml-sig/ \n " \
+          "You can still use the demo, just don't try to create a music catalog"
+
+
 # Snarfzilla stuff:
 import config, jaxml, Help
 from ID3Tag import *
@@ -364,7 +392,7 @@ class MyFrame(wxFrame):
 
 
     def OnMnuHelp(self, event):
-        Help.showHelp(self, Help.BoaHelpFrame, 'help.html') #./snarfzilla.0.0.2/Docs/help.html
+        Help.showHelp(self, Help.BoaHelpFrame, 'index.html') #./snarfzilla.0.0.3/Docs/index.html
 
 
     def OnMnuExit(self,event):
@@ -420,219 +448,6 @@ class panellog(wxPanel):
         self.log.DeleteAllItems()
 
 
-class panelcreate(wxPanel):
-    def __init__(self,parent):
-        wxPanel.__init__(self, parent, -1)
-        self.SetAutoLayout(true)
-
-        row = 25
-        col = 90
-        width = 225
-
-        #self.rembutID=NewId()
-        self.rembut = wxButton(self, 1500, "Select MP3    ", wxPoint(330, row))
-        self.id3but = wxButton(self, 1501, "Write ID3 Tags", wxPoint(330, row+30))
-        self.id3msg = wxStaticText(self, -1, "", wxPoint(430, 55), wxSize(75, 20))
-        self.addbut = wxButton(self, 1502, "Add to Catalog", wxPoint(330, row+60))
-        self.fairtunesbut = wxButton(self, 1504, "Fairtunes", wxPoint(330, row+90))
-
-        EVT_BUTTON(self, 1500, self.fileSelect)
-        EVT_BUTTON(self, 1501, self.writeID3)
-        EVT_BUTTON(self, 1502, self.addCatalog)
-        EVT_BUTTON(self, 1504, self.fairtunes)
-
-        wxStaticText(self, -1, "Filename", wxPoint(5, row), wxSize(75, 20))
-        self.inpFilename = wxTextCtrl(self, 5, "", wxPoint(col, row), wxSize(width, 20))
-        self.inpFilename.SetInsertionPoint(0)
-
-        wxStaticText(self, -1, "Artist", wxPoint(5, row+30), wxSize(75, 20))
-        self.inpArtist = wxTextCtrl(self, 10, "", wxPoint(col, row+30), wxSize(width, 20))
-        self.inpArtist.SetInsertionPoint(0)
-
-        wxStaticText(self, -1, "Song Title", wxPoint(5, row+60), wxSize(75, 20))
-        self.inpTitle = wxTextCtrl(self, 20, "", wxPoint(col, row+60), wxSize(width, 20))
-        self.inpTitle.SetInsertionPoint(0)
-
-        wxStaticText(self, -1, "Album", wxPoint(5, row+90), wxSize(75, 20))
-        self.inpAlbum = wxTextCtrl(self, 30, "", wxPoint(col, row+90), wxSize(width, 20))
-        self.inpAlbum.SetInsertionPoint(0)
-
-        wxStaticText(self, -1, "Comment", wxPoint(5, row+120), wxSize(75, 20))
-        self.inpComment = wxTextCtrl(self, 40, "", wxPoint(col, row+120), wxSize(width, 20))
-        self.inpComment.SetInsertionPoint(0)
-
-        wxStaticText(self, -1, "Genre", wxPoint(5, row+150), wxSize(75, 20))
-        self.inpGenre = wxTextCtrl(self, 50, "", wxPoint(col, row+150), wxSize(width, 20))
-        self.inpGenre.SetInsertionPoint(0)
-
-        wxStaticText(self, -1, "Year", wxPoint(5, row+180), wxSize(75, 20))
-        self.inpYear = wxTextCtrl(self, 60, "", wxPoint(col, row+180), wxSize(width, 20))
-        self.inpYear.SetInsertionPoint(0)
-
-        self.header = wxStaticText(self, -1, "", wxPoint(col, row+210), wxSize(width, 20))
-        self.insbut = wxButton(self, 1505, "Upload Catalog to Freenet", wxPoint(90, row+250))
-        EVT_BUTTON(self, 1505, self.insCatalog)
-
-        wxStaticText(self, -1, "Catalog Name", wxPoint(5, row+280), wxSize(80, 20))
-        self.inpCatalogName = wxTextCtrl(self, 90, "", wxPoint(col, row+280), wxSize(width, 20))
-        self.inpCatalogName.SetInsertionPoint(0)
-
-        self.catListID=NewId()
-        self.catList = wxListCtrl(self, self.catListID, wxPoint(10,330), wxDefaultSize,wxLC_REPORT|wxSUNKEN_BORDER)
-        lc = wxLayoutConstraints()
-        lc.top.SameAs(self, wxTop, 330)
-        lc.left.SameAs(self, wxLeft, 10)
-        lc.bottom.SameAs(self, wxBottom, 10)
-        lc.right.SameAs(self, wxRight, 10)
-
-        self.catList.SetConstraints(lc)
-        self.catList.InsertColumn(0, "Artist")
-        self.catList.InsertColumn(1, "Title")
-        self.catList.InsertColumn(2, "Album")
-        self.catList.InsertColumn(3, "Time")
-        self.catList.InsertColumn(4, "Bitrate")
-        #self.catList.SetColumnWidth(0, wxLIST_AUTOSIZE_USEHEADER)
-        self.catList.SetColumnWidth(0, 170)
-        self.catList.SetColumnWidth(1, 170)
-        self.catList.SetColumnWidth(2, 170)
-        self.catList.SetColumnWidth(3, wxLIST_AUTOSIZE_USEHEADER)
-        self.catList.SetColumnWidth(4, wxLIST_AUTOSIZE_USEHEADER)
-        self.playlist = {}
-
-    def insCatalog(self, event):
-        xit = Xml.XML_document()
-        xit.catalog()
-        items = self.playlist.items()
-        for x in range(len(items)):
-            key, data = items[x]
-            xit.song()
-            xit.artist(data[0])
-            xit.title(data[1])
-            xit.album(data[2])
-            xit.time(data[3])
-            xit.filesize(data[4])
-            xit.bitrate(data[5])
-            xit.freq(data[6])
-            xit.genre(data[7])
-            xit.year(data[8])
-            xit.key("freeent:KSK@blah")
-        f = self.inpCatalogName.GetValue()
-        if f == "":
-            filename = "Enter_a_catalog_name_next_time.xml"
-        else:
-            filename = f + ".xml"
-
-        xit._output(filename)
-        self.inpFilename.SetValue("")
-        self.inpArtist.SetValue("")
-        self.inpTitle.SetValue("")
-        self.inpAlbum.SetValue("")
-        self.inpComment.SetValue("")
-        self.inpGenre.SetValue("")
-        self.inpYear.SetValue("")
-        self.playlist = {}
-        self.id3msg.SetLabel("")
-        self.header.SetLabel("")
-        self.catList.ClearAll()
-
-    def addCatalog(self, event):
-        x = len(self.playlist.items())
-
-        artist = self.inpArtist.GetValue()
-        title = self.inpTitle.GetValue()
-        album = self.inpAlbum.GetValue()
-        comment = self.inpComment.GetValue()
-        genre = self.inpGenre.GetValue()
-        year = self.inpYear.GetValue()
-        version = self.i['VERSION']
-        layer = self.i['LAYER']
-        bitrate = self.i['BITRATE']
-        frequency = self.i['FREQUENCY']
-        sec = ("%s" % self.i['SS'])
-        if len(sec) == 1:    # Add leading zero if seconds < 10 so time looks pretty (4:04 instead of 4:4)
-            seconds = ("0%s" % sec)
-        else:
-            seconds = ("%s" % sec)
-        length = "%s:%s" % (self.i['MM'], seconds)
-
-        self.playlist[x+1] = ( artist, title, album, comment, genre, year, version, layer, bitrate, frequency, length )
-        self.catList.InsertStringItem(x, artist)
-        self.catList.SetStringItem(x, 1, title)
-        self.catList.SetStringItem(x, 2, album)
-        self.catList.SetStringItem(x, 3, length)
-        self.catList.SetStringItem(x, 4, "%s Kbps" % (bitrate))
-        self.catList.SetItemData(x, x)
-
-    def writeID3(self, event):
-        filename = self.inpFilename.GetValue()
-        if filename != "":
-            artist = string.rstrip(self.inpArtist.GetValue())
-            title = string.rstrip(self.inpTitle.GetValue())
-            album = string.rstrip(self.inpAlbum.GetValue())
-            comment = string.rstrip(self.inpComment.GetValue())
-            genre = string.rstrip(self.inpGenre.GetValue())
-            year = string.rstrip(self.inpYear.GetValue())
-            tag=ID3Tag(filename)
-            tag.Title(title)
-            tag.Artist(artist)
-            tag.Album(album)
-            tag.Year(year)
-            tag.Genre(genre)
-            tag.Comment(comment)
-
-            tag.Commit()
-            self.id3msg.SetLabel("Tags written")
-        else:
-            self.id3msg.SetLabel("You need a filename.")
-
-
-    def fileSelect(self, event):
-        dlg = wxFileDialog(self, "Choose a file", ".", "", "*.mp3", wxOPEN|wxMULTIPLE)
-        if dlg.ShowModal() == wxID_OK:
-            for path in dlg.GetPaths():
-                self.inpFilename.SetValue(path)
-                dlg.Destroy()
-        self.id3msg.SetLabel("")
-        #MP3 header info:
-        self.i = mp3info(path)
-        self.header.SetLabel('MPEG%d, L%d %dKbps %dHz Stereo' % \
-                              (self.i['VERSION'], self.i['LAYER'], self.i['BITRATE'], self.i['FREQUENCY']))
-
-        ### i['STEREO'] # Not implemented, I faked it
-        ### i['MODE'] # Not implemented
-        ### i['COPYRIGHT'] # Not implemented
-
-        wxStaticText(self, -1, "              ", wxPoint(430, 55), wxSize(75, 20)) # Ummm this is cheesier than Velveeta.
-
-        # ID3 v1 info:
-        try:
-            tag = ID3Tag(path)
-            tag.Read()
-            self.inpArtist.SetValue(string.rstrip(tag.theArtist()))
-            self.inpTitle.SetValue(string.rstrip(tag.theTitle()))
-            self.inpAlbum.SetValue(string.rstrip(tag.theAlbum()))
-            self.inpComment.SetValue(string.rstrip(tag.theComment()))
-            self.inpGenre.SetValue(string.rstrip(tag.theGenre()))
-            self.inpYear.SetValue(string.rstrip(tag.theYear()))
-        except:
-            self.inpArtist.SetValue("")
-            self.inpTitle.SetValue("")
-            self.inpAlbum.SetValue("")
-            self.inpComment.SetValue("")
-            self.inpGenre.SetValue("")
-            self.inpYear.SetValue("")
-
-    def fairtunes(self, event):
-        artist = self.inpArtist.GetValue()
-        nice_artist = string.joinfields(string.splitfields(artist,' '),'%20') # Fix this: theres some url pretty deal in Python?
-        #print nice_artist
-        # Added client=snarfzilla per Matt Goyer at Fairtunes
-        url = 'http://www.fairtunes.com/bare_search.jsp?searchTerms=' + nice_artist + '&action=dosearch&client=snarfzilla'
-        #print url
-        doit = browserloc + " '" + url + "' &"
-        os.system(doit)
-
-
 class panelkeybrowser(wxPanel):
     def __init__(self, parent, list):
         wxPanel.__init__(self, parent, -1)
@@ -750,6 +565,13 @@ class paneldownload(wxPanel):
         EVT_RIGHT_UP(self.list, self.OnRightClick)
         EVT_RIGHT_DOWN(self.list, self.OnRightDown)
 
+        self.lblRequesting = wxStaticText(self, -1, 'Saving as: ') # Placeholder for "Saving as: "
+        self.inpRequesting = wxStaticText(self, -1, '')
+        box3top = wxBoxSizer(wxHORIZONTAL)
+        box3top.Add(self.lblRequesting, 0, wxLEFT)
+        box3top.Add(self.inpRequesting, 1, wxEXPAND|wxALL)
+        self.lblRequesting.SetLabel('')
+
         #----------------------------------------------------------------------------------
         # Filename input  box  wxListCtrl (Box 3)
         #----------------------------------------------------------------------------------
@@ -789,12 +611,12 @@ class paneldownload(wxPanel):
         box4.Add(self.sndBtn, 0, wxLEFT, 5)
         #box4.Add(self.termBtn, 0, wxLEFT, 5)
 
-        #Put everything into a sizer
 
         sizer = wxBoxSizer(wxVERTICAL)
         sizer.Add(box1, 0, wxEXPAND|wxALL, 0)
         sizer.Add(self.out, 1, wxEXPAND|wxALL, 0)
         sizer.Add(box2, 1, wxEXPAND|wxALL, 0)
+        sizer.Add(box3top, 0, wxEXPAND|wxALL, 5)
         sizer.Add(box3, 0, wxEXPAND|wxALL, 5)
         sizer.Add(box4, 0, wxEXPAND|wxALL, 5)
         self.SetSizer(sizer)
@@ -805,10 +627,33 @@ class paneldownload(wxPanel):
         # It will also check the que to see if downloads are finished and start
         # the next file download etc.
         self.timer = wxPyTimer(self.Pull)
-        self.timer.Start(2000)
+        self.timer.Start(4000) #4 seconds
         self.Pull()
 
     def Pull(self):
+        #Check to see if there are no downloads in progress, if so start next queued key downloading
+        """nbr = self.list.GetItemCount() - 1
+        busy = 0
+        while nbr > -1:
+            item = self.list.GetItem(nbr, 1)
+            col1 = item.GetText()
+            nbr = nbr -1
+            if col1 == "Requesting" or col1 == "Done":
+                pass
+            else:
+                if col1 == "Queued":
+                    item = self.list.GetItem(nbr, 1)
+                    n = self.list.GetItemText(nbr)
+                    self.inp.SetValue(n)
+        #print item.m_text, item.m_itemId , self.list.GetItemData(self.currentItem)
+        #print item.m_data
+        #print self.list.GetItemText(self.currentItem)
+
+                    self.list.DeleteItem(nbr) #Delete the item we're on because it gets added again in OnSendText
+                    self.OnSendText(self) # Send to download tab, start downloading or be queued
+                    nbr = -1 # Exit while loop"""
+
+        #Check to see if there is any text output from download.py to display in out window
         if self.process is not None:
             stream = self.process.GetInputStream()
             if not stream.eof():
@@ -816,6 +661,7 @@ class paneldownload(wxPanel):
                 self.out.AppendText(text)
 
     def OnSmartNames(self, event):
+        # Smart Names Checkbox check or uncheck
         n = self.smartnames.GetValue()
         if n == 1:
             self.inpFilename.Enable(false)
@@ -837,25 +683,42 @@ class paneldownload(wxPanel):
         self.x = event.GetX()
         self.y = event.GetY() + 250
 
-        self.log.add("OnRightClick %s\n" % self.list.GetItemText(self.currentItem))
+        #self.log.add("OnRightClick %s\n" % self.list.GetItemText(self.currentItem))
         menu = wxMenu()
         tPopupID1 = 0
         tPopupID2 = 1
         menu.Append(tPopupID1, "Force download")
-        #menu.Append(tPopupID2, "Launch (MP3s w/ XMMS)") # Whoa funky bug prints \ instead of /
-        menu.Append(tPopupID2, 'Launch (MP3s w/ XMMS)') # Whoa funky bug prints \ instead of /
-        EVT_MENU(self, tPopupID1, self.OnPopupOne)
+        #menu.Append(tPopupID2, 'Launch (MP3s w/ XMMS)') # Whoa funky bug prints \ instead of /
+        menu.Append(tPopupID2, 'Launch (MP3s with XMMS)')
+        EVT_MENU(self, tPopupID1, self.OnForceDownload)
         EVT_MENU(self, tPopupID2, self.OnLaunch)
         self.PopupMenu(menu, wxPoint(self.x, self.y))
         menu.Destroy()
         event.Skip()
 
-    def OnPopupOne(self, event):
+    def OnForceDownload(self, event):
+        nbr = self.list.GetItemCount() - 1
+        busy = 0
+        while nbr > -1:
+            item = self.list.GetItem(nbr, 1)
+            col1 = item.GetText()
+            nbr = nbr -1
+            if col1 == "Requesting":
+                dlg = wxMessageDialog(self, 'Only one download at a time.' ,
+                              'Please be patient', wxOK | wxICON_INFORMATION)
+                dlg.ShowModal()
+                dlg.Destroy()
+
         n = self.list.GetItemText(self.currentItem)
         self.inp.SetValue(n)
         i = self.currentItem
-        self.list.DeleteItem(i)
-        self.OnSendText(self)
+
+        #print item.m_text, item.m_itemId , self.list.GetItemData(self.currentItem)
+        #print item.m_data
+        #print self.list.GetItemText(self.currentItem)
+
+        self.list.DeleteItem(i) #Delete the item we're on because it gets added again in OnSendText
+        self.OnSendText(self) # Send to download tab, start downloading or be queued
 
     def OnLaunch(self, event):
         keyname = self.list.GetItemText(self.currentItem)
@@ -870,36 +733,59 @@ class paneldownload(wxPanel):
             self.process = None
 
     def OnSendText(self, evt):
-        cmd = "python download.py"
-        self.process = wxProcess(self)
-        self.process.Redirect();
-        pid = wxExecute(cmd, false, self.process)
-        self.log.add('Starting download.py as pid %s\n' % (pid))
         keyname = self.inp.GetValue()
-        filename = self.inpFilename.GetValue()
-        v = self.detail.GetValue()
-        verbose = ('%s' % v)
-        s = self.smartnames.GetValue()
-        # If smartnames checked, make filename from key like KSK@mp3/britney/iloverob.mp3 to iloverob.mp3
-        if s == 1:
-            if string.find(keyname, '/') != -1:
-                dirname, filename = os.path.split(keyname)
-            else:
-                filename = keyname # plain kay like 'test'
-        self.inpFilename.SetValue(filename)
-        self.process.GetOutputStream().write(verbose + '\n')
-        self.process.GetOutputStream().write(filename + '\n')
-        self.process.GetOutputStream().write(keyname + '\n')
-        #print verbose, filename, keyname
-        self.out.AppendText("Requesting key: \n %s \n\n" % keyname)
-        self.out.AppendText("Saving as file: \n %s \n" % filename)
-        self.inp.Enable(false)
-        self.inpFilename.Enable(false)
-        self.sndBtn.Enable(false)
-        #self.termBtn.Enable(true)
+        self.lblRequesting.SetLabel('Saving as: ')
+        #If there are zero keys, start download right away, else mark Queued
+        nbr = self.list.GetItemCount() - 1
+        busy = 0
+        while nbr > -1:
+            item = self.list.GetItem(nbr, 1)
+            col1 = item.GetText()
+            nbr = nbr -1
+            if col1 == "Requesting":
+                busy = 1
+
         pos=self.list.GetItemCount()
-        self.list.InsertStringItem(pos, keyname)
-        self.list.SetStringItem(pos, 1, "Requesting")
+        if busy == 0: # Not requesting, so start downloading, only one at a time though for now
+
+            self.list.InsertStringItem(pos, keyname)
+            self.list.SetStringItem(pos, 1, "Requesting")
+
+            cmd = "python download.py"
+            self.process = wxProcess(self)
+            self.process.Redirect();
+            pid = wxExecute(cmd, false, self.process)
+            self.log.add('Starting download.py as pid %s\n' % (pid))
+
+            filename = self.inpFilename.GetValue()
+
+            v = self.detail.GetValue()
+            verbose = ('%s' % v)
+            s = self.smartnames.GetValue()
+            # If smartnames checked, make filename from key like KSK@mp3/britney/iloverob.mp3 to iloverob.mp3
+            if s == 1:
+                if string.find(keyname, '/') != -1:
+                    dirname, filename = os.path.split(keyname)
+                else:
+                    filename = keyname # plain kay like 'test'
+            self.inpRequesting.SetLabel("./download/%s" % filename)
+
+            self.process.GetOutputStream().write(verbose + '\n')
+            self.process.GetOutputStream().write(filename + '\n')
+            self.process.GetOutputStream().write(keyname + '\n')
+            #print verbose, filename, keyname
+            self.out.AppendText("Requesting key: \n %s \n\n" % keyname)
+            self.out.AppendText("Saving as file: \n %s \n" % filename)
+            self.inp.SetValue('')
+            #self.inp.Enable(true)
+            #self.inpFilename.Enable(true)
+            #self.sndBtn.Enable(true)
+            #self.termBtn.Enable(true)
+        else:
+            self.list.InsertStringItem(pos, keyname)
+            self.list.SetStringItem(pos, 1, "Queued")
+            self.inp.SetValue('')
+            self.inpFilename.SetValue('')
 
     def add(self, txt):
         pos=self.list.GetItemCount()
@@ -915,6 +801,9 @@ class paneldownload(wxPanel):
         self.inp.SetFocus()
         self.log.add('OnCloseStream')
 
+# OnIdle used to update the GUI with the download.py output
+# but it only worked when you moved the mouse or hit a key
+# def Pull has replaced this and is a 2 second timer
 
 #       def OnIdle(self, evt):
 #        if self.process is not None:
@@ -928,26 +817,26 @@ class paneldownload(wxPanel):
     #----------------------------------------------------------------
 
     def OnProcessEnded(self, evt):
+
         stream = self.process.GetInputStream()
         if not stream.eof():
             text = stream.read()
             self.process.Destroy()
             self.process = None
 
-            filename = self.inpFilename.GetValue()
-
-            self.inp.Enable(true)
-            #self.inpFilename.Enable(true)
+            filename = self.inpRequesting.GetLabel()
             self.inp.SetFocus()
             self.sndBtn.Enable(true)
-            #self.termBtn.Enable(false)
-            self.inp.SetValue('')
-            self.inpFilename.SetValue('')
+            #self.inp.SetValue('')
+            #self.inpFilename.SetValue('')
             pos=self.list.GetItemCount()
             nbr_items = pos - 1
             #self.list.SetColumnWidth(0, wxLIST_AUTOSIZE)
             #self.list.EnsureVisible(pos) ### This makes it hang.
-
+            #print "Snarfie"
+            #print text
+            #print len(text)
+            #print string.find(text, "asdfas")
             if text[-1] == '*':
                 failed = 1
             else:
@@ -958,7 +847,7 @@ class paneldownload(wxPanel):
                 status = "Done"
 
             try:
-                a = os.stat("./download/%s" % filename)
+                a = os.stat("%s" % filename)
                 f = a[6]
             except:
                 f = 0
@@ -970,8 +859,8 @@ class paneldownload(wxPanel):
             self.list.SetStringItem(nbr_items, 2, filesize)
             self.out.AppendText(text)
             self.out.AppendText("\n")
-
-
+            self.inpRequesting.SetLabel("")
+            self.lblRequesting.SetLabel("")
 
 #------------------------------------------------------------------------------------------
 # Upload Panel
@@ -1144,14 +1033,12 @@ class panelupload(wxPanel):
         #item = self.list.GetItem(self.currentItem)
         print item.m_text, item.m_itemId # , self.list.GetItemData(self.currentItem)
 
-
     def __del__(self):
         if self.process is not None:
             self.process.Detach()
             self.process.CloseOutput()
             self.process = None
 
-
     def OnSendText(self, evt):
         cmd = "python upload.py"
 
@@ -1189,7 +1076,6 @@ class panelupload(wxPanel):
 
 
     def OnIdle(self, evt):
-        #print "onidle"
         if self.process is not None:
             stream = self.process.GetInputStream()
             if not stream.eof():
@@ -1198,6 +1084,10 @@ class panelupload(wxPanel):
 
 
     def OnProcessEnded(self, evt):
+
+        # There is something funky going on here. Hats off to whoever figures it out.
+        # You can't send anything to other tabs -  self.log.add("foo") makes it hang
+
         stream = self.process.GetInputStream()
         if not stream.eof():
             text = stream.read()
@@ -1315,7 +1205,6 @@ class panelbrowser(wxPanel):
         dlg.Destroy()
 
 
-
 class MyHtmlWindow(wxHtmlWindow):
     def __init__(self, parent):
         wxHtmlWindow.__init__(self, parent)
@@ -1377,14 +1266,225 @@ class MyHtmlWindow(wxHtmlWindow):
             output.close()
             self.LoadPage('./htdocs/cache/%s' % urlcache)
 
-        # Virtuals in the base class have been renamed with base_ on the front.
-
-
 
     def OnSetTitle(self, title):
         #self.log.add('OnSetTitle: %s\n' % title)
         self.base_OnSetTitle(title)
 
+class panelcreate(wxPanel):
+    def __init__(self,parent):
+        wxPanel.__init__(self, parent, -1)
+        self.SetAutoLayout(true)
+
+        # Ok, so maybe this was my first window and I didn't know about wxSizers...
+        row = 25
+        col = 90
+        width = 225
+
+        #self.rembutID=NewId()
+        self.rembut = wxButton(self, 1500, "Select MP3    ", wxPoint(330, row))
+        self.id3but = wxButton(self, 1501, "Write ID3 Tags", wxPoint(330, row+30))
+        self.id3msg = wxStaticText(self, -1, "", wxPoint(430, 55), wxSize(75, 20))
+        self.addbut = wxButton(self, 1502, "Add to Catalog", wxPoint(330, row+60))
+        self.fairtunesbut = wxButton(self, 1504, "Fairtunes", wxPoint(330, row+90))
+
+        EVT_BUTTON(self, 1500, self.fileSelect)
+        EVT_BUTTON(self, 1501, self.writeID3)
+        EVT_BUTTON(self, 1502, self.addCatalog)
+        EVT_BUTTON(self, 1504, self.fairtunes)
+
+        wxStaticText(self, -1, "Filename", wxPoint(5, row), wxSize(75, 20))
+        self.inpFilename = wxTextCtrl(self, 5, "", wxPoint(col, row), wxSize(width, 20))
+        self.inpFilename.SetInsertionPoint(0)
+
+        wxStaticText(self, -1, "Artist", wxPoint(5, row+30), wxSize(75, 20))
+        self.inpArtist = wxTextCtrl(self, 10, "", wxPoint(col, row+30), wxSize(width, 20))
+        self.inpArtist.SetInsertionPoint(0)
+
+        wxStaticText(self, -1, "Song Title", wxPoint(5, row+60), wxSize(75, 20))
+        self.inpTitle = wxTextCtrl(self, 20, "", wxPoint(col, row+60), wxSize(width, 20))
+        self.inpTitle.SetInsertionPoint(0)
+
+        wxStaticText(self, -1, "Album", wxPoint(5, row+90), wxSize(75, 20))
+        self.inpAlbum = wxTextCtrl(self, 30, "", wxPoint(col, row+90), wxSize(width, 20))
+        self.inpAlbum.SetInsertionPoint(0)
+
+        wxStaticText(self, -1, "Comment", wxPoint(5, row+120), wxSize(75, 20))
+        self.inpComment = wxTextCtrl(self, 40, "", wxPoint(col, row+120), wxSize(width, 20))
+        self.inpComment.SetInsertionPoint(0)
+
+        wxStaticText(self, -1, "Genre", wxPoint(5, row+150), wxSize(75, 20))
+        self.inpGenre = wxTextCtrl(self, 50, "", wxPoint(col, row+150), wxSize(width, 20))
+        self.inpGenre.SetInsertionPoint(0)
+
+        wxStaticText(self, -1, "Year", wxPoint(5, row+180), wxSize(75, 20))
+        self.inpYear = wxTextCtrl(self, 60, "", wxPoint(col, row+180), wxSize(width, 20))
+        self.inpYear.SetInsertionPoint(0)
+
+        self.header = wxStaticText(self, -1, "", wxPoint(col, row+210), wxSize(width, 20))
+        self.insbut = wxButton(self, 1505, "Upload Catalog to Freenet", wxPoint(90, row+250))
+        EVT_BUTTON(self, 1505, self.insCatalog)
+
+        wxStaticText(self, -1, "Catalog Name", wxPoint(5, row+280), wxSize(80, 20))
+        self.inpCatalogName = wxTextCtrl(self, 90, "", wxPoint(col, row+280), wxSize(width, 20))
+        self.inpCatalogName.SetInsertionPoint(0)
+
+        self.catListID=wxNewId()
+        self.catList = wxListCtrl(self, self.catListID, wxPoint(10,330), wxDefaultSize,wxLC_REPORT|wxSUNKEN_BORDER)
+        lc = wxLayoutConstraints()
+        lc.top.SameAs(self, wxTop, 330)
+        lc.left.SameAs(self, wxLeft, 10)
+        lc.bottom.SameAs(self, wxBottom, 10)
+        lc.right.SameAs(self, wxRight, 10)
+
+        self.catList.SetConstraints(lc)
+        self.catList.InsertColumn(0, "Artist")
+        self.catList.InsertColumn(1, "Title")
+        self.catList.InsertColumn(2, "Album")
+        self.catList.InsertColumn(3, "Time")
+        self.catList.InsertColumn(4, "Bitrate")
+        #self.catList.SetColumnWidth(0, wxLIST_AUTOSIZE_USEHEADER)
+        self.catList.SetColumnWidth(0, 170)
+        self.catList.SetColumnWidth(1, 170)
+        self.catList.SetColumnWidth(2, 170)
+        self.catList.SetColumnWidth(3, wxLIST_AUTOSIZE_USEHEADER)
+        self.catList.SetColumnWidth(4, wxLIST_AUTOSIZE_USEHEADER)
+        self.playlist = {}
+
+    def insCatalog(self, event):
+        xit = Xml.XML_document()
+        xit.catalog()
+        items = self.playlist.items()
+        for x in range(len(items)):
+            key, data = items[x]
+            xit.song()
+            xit.artist(data[0])
+            xit.title(data[1])
+            xit.album(data[2])
+            xit.time(data[3])
+            xit.filesize(data[4])
+            xit.bitrate(data[5])
+            xit.freq(data[6])
+            xit.genre(data[7])
+            xit.year(data[8])
+            xit.key("freeent:KSK@blah")
+        f = self.inpCatalogName.GetValue()
+        if f == "":
+            filename = "Enter_a_catalog_name_next_time.xml"
+        else:
+            filename = f + ".xml"
+
+        xit._output(filename)
+        self.inpFilename.SetValue("")
+        self.inpArtist.SetValue("")
+        self.inpTitle.SetValue("")
+        self.inpAlbum.SetValue("")
+        self.inpComment.SetValue("")
+        self.inpGenre.SetValue("")
+        self.inpYear.SetValue("")
+        self.playlist = {}
+        self.id3msg.SetLabel("")
+        self.header.SetLabel("")
+        self.catList.ClearAll()
+
+    def addCatalog(self, event):
+        x = len(self.playlist.items())
+
+        artist = self.inpArtist.GetValue()
+        title = self.inpTitle.GetValue()
+        album = self.inpAlbum.GetValue()
+        comment = self.inpComment.GetValue()
+        genre = self.inpGenre.GetValue()
+        year = self.inpYear.GetValue()
+        version = self.i['VERSION']
+        layer = self.i['LAYER']
+        bitrate = self.i['BITRATE']
+        frequency = self.i['FREQUENCY']
+        sec = ("%s" % self.i['SS'])
+        if len(sec) == 1:    # Add leading zero if seconds < 10 so time looks pretty (4:04 instead of 4:4)
+            seconds = ("0%s" % sec)
+        else:
+            seconds = ("%s" % sec)
+        length = "%s:%s" % (self.i['MM'], seconds)
+
+        self.playlist[x+1] = ( artist, title, album, comment, genre, year, version, layer, bitrate, frequency, length )
+        self.catList.InsertStringItem(x, artist)
+        self.catList.SetStringItem(x, 1, title)
+        self.catList.SetStringItem(x, 2, album)
+        self.catList.SetStringItem(x, 3, length)
+        self.catList.SetStringItem(x, 4, "%s Kbps" % (bitrate))
+        self.catList.SetItemData(x, x)
+
+    def writeID3(self, event):
+        filename = self.inpFilename.GetValue()
+        if filename != "":
+            artist = string.rstrip(self.inpArtist.GetValue())
+            title = string.rstrip(self.inpTitle.GetValue())
+            album = string.rstrip(self.inpAlbum.GetValue())
+            comment = string.rstrip(self.inpComment.GetValue())
+            genre = string.rstrip(self.inpGenre.GetValue())
+            year = string.rstrip(self.inpYear.GetValue())
+            tag=ID3Tag(filename)
+            tag.Title(title)
+            tag.Artist(artist)
+            tag.Album(album)
+            tag.Year(year)
+            tag.Genre(genre)
+            tag.Comment(comment)
+
+            tag.Commit()
+            self.id3msg.SetLabel("Tags written")
+        else:
+            self.id3msg.SetLabel("You need a filename.")
+
+
+    def fileSelect(self, event):
+        dlg = wxFileDialog(self, "Choose a file", ".", "", "*.mp3", wxOPEN|wxMULTIPLE)
+        if dlg.ShowModal() == wxID_OK:
+            for path in dlg.GetPaths():
+                self.inpFilename.SetValue(path)
+                dlg.Destroy()
+        self.id3msg.SetLabel("")
+        #MP3 header info:
+        self.i = mp3info(path)
+        self.header.SetLabel('MPEG%d, L%d %dKbps %dHz Stereo' % \
+                              (self.i['VERSION'], self.i['LAYER'], self.i['BITRATE'], self.i['FREQUENCY']))
+
+        ### i['STEREO'] # Not implemented, I faked it
+        ### i['MODE'] # Not implemented
+        ### i['COPYRIGHT'] # Not implemented
+
+        wxStaticText(self, -1, "              ", wxPoint(430, 55), wxSize(75, 20)) # Ummm this is cheesier than Velveeta.
+
+        # ID3 v1 info:
+        try:
+            tag = ID3Tag(path)
+            tag.Read()
+            self.inpArtist.SetValue(string.rstrip(tag.theArtist()))
+            self.inpTitle.SetValue(string.rstrip(tag.theTitle()))
+            self.inpAlbum.SetValue(string.rstrip(tag.theAlbum()))
+            self.inpComment.SetValue(string.rstrip(tag.theComment()))
+            self.inpGenre.SetValue(string.rstrip(tag.theGenre()))
+            self.inpYear.SetValue(string.rstrip(tag.theYear()))
+        except:
+            self.inpArtist.SetValue("")
+            self.inpTitle.SetValue("")
+            self.inpAlbum.SetValue("")
+            self.inpComment.SetValue("")
+            self.inpGenre.SetValue("")
+            self.inpYear.SetValue("")
+
+    def fairtunes(self, event):
+        artist = self.inpArtist.GetValue()
+        nice_artist = string.joinfields(string.splitfields(artist,' '),'%20') # Fix this: theres some url pretty deal in Python?
+        #print nice_artist
+        # Added client=snarfzilla per Matt Goyer at Fairtunes
+        url = 'http://www.fairtunes.com/bare_search.jsp?searchTerms=' + nice_artist + '&action=dosearch&client=snarfzilla'
+        #print url
+        doit = browserloc + " '" + url + "' &"
+        os.system(doit)
+
+
 
 class panelkeyring(wxPanel):
     def __init__(self,parent):
@@ -1415,7 +1515,7 @@ class panelkeyring(wxPanel):
         lc.height.AsIs()
         lc.width.AsIs()
         but.SetConstraints(lc);
-        self.keylist.InsertStringItem(0, "test")
+        #self.keylist.InsertStringItem(0, "test")
 
     def add(self,txt):
         pos=self.keylist.GetItemCount()
@@ -1433,7 +1533,7 @@ class panelkeyring(wxPanel):
 
 class MyApp(wxApp):
     def OnInit(self):
-        frame=MyFrame(None,-1,"Snarfzilla - The Freenet Monster")
+        frame=MyFrame(None,-1,"Snarfzilla - The Freenet Monster 0.0.3")
         frame.Show(true)
         self.SetTopWindow(frame)
         return true
